<?php
/**
 * Perforce Swarm
 *
 * @copyright   2013-2020 Perforce Software. All rights reserved.
 * @license     Please see LICENSE.txt in top-level readme folder of this distribution.
 * @version     2020.1/2030377
 */

namespace Application\Validator;

use Laminas\Validator\AbstractValidator;

/**
 * Class ArrayValuesValidator. Validate a value against a known list of valid values
 * @package Application\Validator
 */
class ArrayValuesValidator extends AbstractValidator
{
    const CASE_SENSITIVE = 'case_sensitive';
    protected $translator;
    protected $validValues;
    protected $field;
    protected $errorKey;
    protected $options;

    /**
     * ArrayValuesValidator constructor.
     * @param mixed     $translator     to translate messages
     * @param array     $validValues    values that are valid
     * @param string    $errorKey       key for any error message
     * @param string    $field          the field being validated, will be referenced on the error message
     * @param array     $options        options for the validation. Supports 'case_sensitive' = true|false to be
     *                                  used for string comparisons. Default is a case insensitive match
     */
    public function __construct($translator, array $validValues, string $errorKey, string $field, array $options = [])
    {
        parent::__construct();
        $this->translator  = $translator;
        $this->validValues = $validValues;
        $this->field       = $field;
        $this->errorKey    = $errorKey;
        $options          += [
            self::CASE_SENSITIVE => false
        ];
        $this->options     = $options;
    }

    /**
     * Validate the value against valid values. If invalid error messages will be set up in the form:
     *
     * ['messages'][<errorKey>] = 'Invalid [<field>], must be one of [<validValues>]'
     * If the value is a string the comparison will be case insensitive
     *
     * @param mixed $value  value to validate
     * @return bool true if valid
     */
    public function isValid($value)
    {
        $valid = true;
        // Case insensitive comparison for string
        if (is_string($value)) {
            $valid = $this->options[self::CASE_SENSITIVE] === true
                ? in_array($value, $this->validValues)
                : in_array(strtolower($value), array_map('strtolower', $this->validValues));
        } elseif (!in_array($value, $this->validValues, true)) {
            $valid = false;
        }
        if ($valid === false) {
            $this->abstractOptions['messages'][$this->errorKey] =
                $this->translator->t(
                    "Invalid %s [%s], must be one of [%s]",
                    [
                        $this->field, is_array($value) ? 'array' : $value,
                        implode(', ', $this->validValues)
                    ]
                );
        }
        return $valid;
    }
}
