<?php
/**
 * Perforce Swarm
 *
 * @copyright   2013-2020 Perforce Software. All rights reserved.
 * @license     Please see LICENSE.txt in top-level readme folder of this distribution.
 * @version     2020.1/2030377
 */
namespace Changes\Controller;

use Api\Controller\AbstractRestfulController;
use Application\Config\Services;
use Application\Connection\ConnectionFactory;
use Application\Filter\Preformat;
use Application\Model\IModelDAO;
use Application\Permissions\Exception\ForbiddenException;
use Application\Permissions\Permissions;
use Jobs\Model\IJob;
use Laminas\Http\Response;
use P4\Spec\Exception\NotFoundException as SpecNotFoundException;
use InvalidArgumentException;
use Laminas\View\Model\JsonModel;
use P4\Filter\Utf8 as Utf8Filter;

/**
 * Class ChangeApi
 * @package Changes\Controller
 */
class ChangeApi extends AbstractRestfulController
{
    const DATA_JOBS = 'jobs';

    /**
     * Get the jobs for a change.
     * Example response:
     *   {
     *       "error": null,
     *       "messages": [],
     *       "data": {
     *           "jobs": [
     *               {
     *                   "job": "job000020",
     *                   "link": "/jobs/job000020",
     *                   "fixStatus": "open",
     *                   "description": "Need Project files\n",
     *                   "descriptionMarkdown": "<span class=\"first-line\">Need Project files</span>"
     *              },
     *              ...
     *              ...
     *       ]
     *   }
     *
     * Example error response:
     *   {
     *       "error": <code>,
     *       "messages": [
     *           {
     *               "code": <code>,
     *               "text": "<message>"
     *           }
     *       ],
     *       "data": null
     *   }
     *
     * @return JsonModel
     */
    public function jobsAction()
    {
        $error = null;
        $data  = null;
        $jobs  = [];
        try {
            $this->services->get(Services::CONFIG_CHECK)->enforce([Permissions::AUTHENTICATED]);
            $changeDao = $this->services->get(IModelDAO::CHANGE_DAO);
            $p4        = $this->services->get(ConnectionFactory::P4);
            $id        = $this->getEvent()->getRouteMatch()->getParam('id');
            $change    = $changeDao->fetchById($id, $p4);
            $jobDao    = $this->services->get(IModelDAO::JOB_DAO);
            $jobs      = $jobDao->getJobs($change);
            $preFormat = new Preformat($this->services, $this->getRequest()->getBaseUrl());
            $utf8      = new Utf8Filter;
            foreach ($jobs as &$job) {
                $job[IJob::FIELD_DESCRIPTION_MARKDOWN] = $preFormat->filter($job[IJob::FIELD_DESCRIPTION]);
                // Filter out any problem characters that may be in the job description
                $job[IJob::FIELD_DESCRIPTION] = $utf8->filter($job[IJob::FIELD_DESCRIPTION]);
            }
        } catch (ForbiddenException $e) {
            $error = $this->buildMessage(Response::STATUS_CODE_403, $e->getMessage());
            $this->getResponse()->setStatusCode(Response::STATUS_CODE_403);
        } catch (SpecNotFoundException $e) {
            // Change id is good but no spec found
            $this->getResponse()->setStatusCode(Response::STATUS_CODE_404);
            $error = $this->buildMessage(Response::STATUS_CODE_404, $e->getMessage());
        } catch (InvalidArgumentException $e) {
            // Change id not correct form, part of the path so use 404
            $this->getResponse()->setStatusCode(Response::STATUS_CODE_404);
            $error = $this->buildMessage(Response::STATUS_CODE_404, $e->getMessage());
        }
        if ($error) {
            $json = $this->error([$error], $this->getResponse()->getStatusCode());
        } else {
            $json = $this->success([self::DATA_JOBS => $jobs]);
        }
        return $json;
    }
}
